@Gtk::Adjustment::ISA = qw(Gtk::Data);
@Gtk::Alignment::ISA = qw(Gtk::Bin);
@Gtk::Arrow::ISA = qw(Gtk::Misc);
@Gtk::AspectFrame::ISA = qw(Gtk::Frame);
@Gtk::Bin::ISA = qw(Gtk::Container);
@Gtk::Box::ISA = qw(Gtk::Container);
@Gtk::Button::ISA = qw(Gtk::Container);
@Gtk::ButtonBox::ISA = qw(Gtk::Box);
@Gtk::CheckButton::ISA = qw(Gtk::ToggleButton);
@Gtk::CheckMenuItem::ISA = qw(Gtk::MenuItem);
@Gtk::ColorSelection::ISA = qw(Gtk::VBox);
@Gtk::ColorSelectionDialog::ISA = qw(Gtk::Window);
@Gtk::Container::ISA = qw(Gtk::Widget);
@Gtk::Curve::ISA = qw(Gtk::DrawingArea);
@Gtk::Data::ISA = qw(Gtk::Object);
@Gtk::Dialog::ISA = qw(Gtk::Window);
@Gtk::DrawingArea::ISA = qw(Gtk::Widget);
@Gtk::Entry::ISA = qw(Gtk::Widget);
@Gtk::EventBox::ISA = qw(Gtk::Bin);
@Gtk::FileSelection::ISA = qw(Gtk::Window);
@Gtk::Fixed::ISA = qw(Gtk::Container);
@Gtk::Frame::ISA = qw(Gtk::Bin);
@Gtk::GammaCurve::ISA = qw(Gtk::VBox);
@Gtk::HBox::ISA = qw(Gtk::Box);
@Gtk::HButtonBox::ISA = qw(Gtk::ButtonBox);
@Gtk::HPaned::ISA = qw(Gtk::Paned);
@Gtk::HRuler::ISA = qw(Gtk::Ruler);
@Gtk::HScale::ISA = qw(Gtk::Scale);
@Gtk::HScrollbar::ISA = qw(Gtk::Scrollbar);
@Gtk::HSeparator::ISA = qw(Gtk::Separator);
@Gtk::Image::ISA = qw(Gtk::Misc);
@Gtk::InputDialog::ISA = qw(Gtk::Window);
@Gtk::Item::ISA = qw(Gtk::Bin);
@Gtk::Label::ISA = qw(Gtk::Misc);
@Gtk::List::ISA = qw(Gtk::Container);
@Gtk::ListItem::ISA = qw(Gtk::Item);
@Gtk::Menu::ISA = qw(Gtk::MenuShell);
@Gtk::MenuBar::ISA = qw(Gtk::MenuShell);
@Gtk::MenuItem::ISA = qw(Gtk::Item);
@Gtk::MenuShell::ISA = qw(Gtk::Container);
@Gtk::Misc::ISA = qw(Gtk::Widget);
@Gtk::Notebook::ISA = qw(Gtk::Container);
@Gtk::Object::ISA = qw();
@Gtk::OptionMenu::ISA = qw(Gtk::Button);
@Gtk::Paned::ISA = qw(Gtk::Container);
@Gtk::Pixmap::ISA = qw(Gtk::Misc);
@Gtk::Preview::ISA = qw(Gtk::Widget);
@Gtk::ProgressBar::ISA = qw(Gtk::Widget);
@Gtk::RadioButton::ISA = qw(Gtk::CheckButton);
@Gtk::RadioMenuItem::ISA = qw(Gtk::CheckMenuItem);
@Gtk::Range::ISA = qw(Gtk::Widget);
@Gtk::Ruler::ISA = qw(Gtk::Widget);
@Gtk::Scale::ISA = qw(Gtk::Range);
@Gtk::Scrollbar::ISA = qw(Gtk::Range);
@Gtk::ScrolledWindow::ISA = qw(Gtk::Container);
@Gtk::Separator::ISA = qw(Gtk::Widget);
@Gtk::Table::ISA = qw(Gtk::Container);
@Gtk::Text::ISA = qw(Gtk::Widget);
@Gtk::ToggleButton::ISA = qw(Gtk::Button);
@Gtk::Tree::ISA = qw(Gtk::Container);
@Gtk::TreeItem::ISA = qw(Gtk::Item);
@Gtk::VBox::ISA = qw(Gtk::Box);
@Gtk::VButtonBox::ISA = qw(Gtk::ButtonBox);
@Gtk::VPaned::ISA = qw(Gtk::Paned);
@Gtk::VRuler::ISA = qw(Gtk::Ruler);
@Gtk::VScale::ISA = qw(Gtk::Scale);
@Gtk::VScrollbar::ISA = qw(Gtk::Scrollbar);
@Gtk::VSeparator::ISA = qw(Gtk::Separator);
@Gtk::Viewport::ISA = qw(Gtk::Bin);
@Gtk::Widget::ISA = qw(Gtk::Object);
@Gtk::Window::ISA = qw(Gtk::Bin);
1;
